{+----------------------------------------------------------------------+
 |  FreeKick :                                                          |
 |                                                                      |
 |       This program provides a memory resident calendar, calculator,  |
 |  and ASCII table that can be called while running another program.   |
 |  Lane Ferris developed the memory resident inline Pascal without     |
 |  which this software would not be possible.                          |
 |                                                                      |
 |  Installation:  Before this program can be made memory resident,     |
 |                 you must install it into RAM.  To do this, just      |
 |                 type the program's name "FREEKICK" when you see      |
 |                 the MSDOS prompt "A>".                               |
 |                      Freekick will respond with its sign-on mess-    |
 |                 age or a message that it can not install itself.     |
 |                                                                      |
 |  Use:           Freekick can be called while running another pro-    |
 |                 gram by pressing the "CTRL" & "\" keys simul-        |
 |                 taneously.  On the Sanyo 555 (no VB), Freekick       |
 |                 can ONLY be called WHILE running another program.    |
 |                                                                      |
 +----------------------------------------------------------------------+}
{ Compile with min/max free dynamic memory set to 150 }

program FreeKick;  { Nov. 24, 1986 }

{$C-}   { turn off CTRL C/S program interruption }

const
   VERSION  = '3.5';
   COMPILED = 'Dec. 27, 1986';

   HOT_KEY  = 11036;   { CTRL\ scan code that triggers interrupt }
   EXIT_KEY =  3103;   { CTRL- scan code frees system RAM        }
   KBD_INT  =   $16;   { MsDos keyboard interrupt                }
   TSR_INT  =   $99;   { temp intr. will point to old kbd intr.  }

{$I Constant.inc }

type
   Str80   = string[80];
   Str20   = string[20];
   CharSet = set of char;
   RegRec  = record           { 8088 CPU registers type definition }
                case boolean of
                   TRUE  : (AX,BX,CX,DX,BP,SI,DI,DS,ES, FLAGS : integer );
                   FALSE : (AL, AH, BL, BH, CL, CH, DL, DH    : byte    );
             end; { Reg Rec }

const  { typed }
   TSR_DSEG : integer = 0;
   TSR_SSEG : integer = 0;
   DOS_SSEG : integer = 0;
   TSR_IP   : integer = 0;
   TSR_CS   : integer = 0;
   TSR_SEG  : integer = 0;
   TSR_OFF  : integer = 0;
   IN_USE   : boolean = FALSE;
   REGS     : RegRec = (AX:0;BX:0;CX:0;DX:0;BP:0;SI:0;DI:0;DS:0;ES:0;FLAGS:0);

var
   SaveRegs : RegRec;
   Segment,
   Offset   : integer;

{$I GetCh.inc    }   { general purpose tools }
{$I Select.inc   }
{$I DrawBox.inc  }
{$I TxtColor.inc }
{$I HexStr.inc   }
{$I IntWindo.inc }
{$I Menu.inc     }

{$I GetDate.inc  }   { calendar related tools }
{$I SetDate.inc  }
{$I MaxDays.inc  }
{$I Calendar.inc }

{$I Calc.inc     }   { calculator }

{$I Ascii.inc    {   { ASCII table }

function VideoMode : byte;
   begin
   FillChar(SaveRegs,SizeOf(SaveRegs),0);
   SaveRegs.AH := $0F;
   Intr($10,SaveRegs);
   VideoMode := SaveRegs.AL;
   end;  { Set Video }

procedure GetVector( IntNum : byte; var S, O : integer );
   begin
   FillChar(SaveRegs,SizeOf(SaveRegs),0);
   SaveRegs.AH := $35;
   SaveRegs.AL := IntNum;
   MsDos(SaveRegs);
   S := SaveRegs.ES;
   O := SaveRegs.BX;
   end;

procedure SetVector( IntNum : byte; S, O : integer );
   begin
   FillChar(SaveRegs,SizeOf(SaveRegs),0);
   with SaveRegs do
      begin
      AH := $25;
      AL := IntNum;
      DS := S;
      DX := O;
      end;
   MsDos(SaveRegs);
   end;

function IntrFree( IntrNum : integer ) : boolean;
   begin
   GetVector(IntrNum,Segment,Offset);
   IntrFree := (Segment + OffSet = 0) or  { IBM free interrupt     }
               (Segment = 64) and
               (Offset = 9435) or         { Sanyo with Video Board }
               (OffSet = 2763);           { standard Sanyo 555     }
   end;  { Interrupt free }

procedure FreeMemory( WhatBlock : integer );
   begin
   SaveRegs.AH := $49;
   SaveRegs.ES := WhatBlock;
   MsDos(SaveRegs);
   end;

procedure TSR_Exit;
   begin
   GetVector(TSR_INT,Segment,Offset);
   SetVector(KBD_INT,Segment,Offset);
   SetVector(TSR_INT,TSR_SEG,TSR_OFF);
   FreeMemory(MemW[Cseg:$2C]);
   FreeMemory(Cseg);
   Halt;
   end;   { TSR exit }

function ConfirmExit : boolean;
   var
      Ch : char;
   begin
   OpenWindo(1,1,45,5,'',White,Red);
   Select(3,2,'Remove Freekick from memory (Y/N) ?>',['Y','N'],Ch);
   CloseWindo;
   ConfirmExit := (Ch = 'Y');
   end; { Confirm Exit }

procedure UseMenu;
   const
      MAIN      : MenuRec = ( Title     : 'Free Kick';
                              Item      : (' Calendar',
                                           'Calculator',
                                           'ASCII table',
                                           '   Exit',
                                           '','','','','','');
                              Choices   : ['1'..'4',ESC];
                              Ch        : NUL;
                              ItemCount : 0;
                              Mx        : 30;
                              My        : 10;
                              ForColor  : Black;
                              BakColor  : LightGray;
                              Width     : 0;
                              Depth     : 0;
                              DefltPos  : 1;
                              CurntPos  : 0;
                              ExitCh    : '4';
                              Exit      : FALSE      );
   begin  { Use Menu }

   repeat
      begin
      OpenMenu(MAIN);
      ReadMenu(MAIN);
      CloseMenu(MAIN);
      case (MAIN.Ch) of
         '1' : Calendar;
         '2' : Calculator;
         '3' : AsciiTable;
         end;
      end;
   until (MAIN.Exit);
   end;   { Use Menu }

procedure TSR_SignOn;
   begin
   ClrScr;
   TxtColor(White,Red);
   DrawBox(1,1,40,9);
   window(2,2,39,8);
   ClrScr;
   writeln(' FreeKick','':17,'Version ',VERSION);
   gotoxy(19,2); writeln('MS-DOS 2.11 generic');
   writeln('':24,COMPILED);
   writeln(LF,' BY:  Lane Ferris & Don Bowen',LF);
   write('Public Domain Software':30);
   window(1,1,80,25);
   TxtColor(LightGray,Black);
   gotoxy(1,11);
   writeln('Press  CTRL \  to activate.');
   writeln('       CTRL -  to remove from memory.');
   end;  { Sign On }

procedure ProcessIntr;
   begin
   Inline ($80/$FC/$00/$74/$07/$5D/$5D/$2E/$FF/$2E/TSR_IP/$FA/$55/$BD/REGS/
           $2E/$89/$46/$00/$2E/$89/$5E/$02/$2E/$89/$4E/$04/$2E/$89/$56/$06/
           $2E/$8F/$46/$08/$2E/$89/$76/$0A/$2E/$89/$7E/$0C/$2E/$8C/$5E/$0E/
           $2E/$8C/$46/$10/$9C/$2E/$8F/$46/$12/$2E/$80/$3E/IN_USE/$01/$74/
           $57/$2E/$8C/$16/DOS_SSEG/$8C/$D6/$8E/$C6/$2E/$8E/$16/TSR_SSEG/
           $2E/$8E/$1E/TSR_DSEG/$2E/$3B/$36/TSR_SSEG/$89/$E6/$74/$05/$3E/
           $8B/$36/$74/$01/$87/$F4/$2E/$FF/$76/$00/$2E/$FF/$76/$02/$2E/$FF/
           $76/$04/$2E/$FF/$76/$06/$2E/$FF/$76/$0A/$2E/$FF/$76/$0C/$2E/$FF/
           $76/$0E/$2E/$FF/$76/$10/$B9/>$0028/$26/$FF/$34/$46/$46/$E2/$F9/
           $2E/$8E/$16/TSR_SSEG/$56/$2E/$8C/$5E/$0E/$FB                    );
   Intr(TSR_INT,REGS);
   if not (IN_USE) then
      if (REGS.AX = HOT_KEY) or (REGS.AX = EXIT_KEY) then
         begin
         if (VideoMode in [2,7]) then
            TextMode(BW80)
         else
            TextMode(C80);
         IN_USE  := TRUE;
         if (REGS.AX = HOT_KEY) then
            UseMenu
         else
            if (ConfirmExit) then
               TSR_Exit;
         REGS.AX := $1D00;
         IN_USE  := FALSE;
         end;
   inline($BD/REGS/$2E/$8B/$46/$00/$2E/$8B/$5E/$02/$2E/$8B/$4E/$04/
          $2E/$8B/$56/$06/$2E/$8B/$76/$0A/$2E/$8B/$7E/$0C/$2E/$8E/
          $5E/$0E/$2E/$8E/$46/$10/$2E/$FF/$76/$12/$9D/$2E/$80/$3E/
          IN_USE/$01/$74/$23/$FA /$5E/$B9/>$0028/$2E/$8E/$06/DOS_SSEG/
          $4E/$4E/$26/$8F/$04/$E2/$F9/$89/$F5/$07/$1F/$5F/$5E/$5A/
          $59/$5B/$44/$44/$89/$EC/$2E/$8E/$16/DOS_SSEG/$5D/$BD/REGS/
          $2E/$FF/$76/$12/$9D/$5D/$FB/$CA/$02/$00                     );
   end;  { Process Intr }

procedure TermStayRes( Space : integer );
   begin
   SaveRegs.AH := $31;
   SaveRegs.Dx := Space; ;     { Prog_Size from Allocation Blk}
   MsDos(SaveRegs);
   end;

begin
if (VideoMode in [2,7]) then
   TextMode(BW80)
else
   TextMode(C80);
if not (IntrFree(TSR_INT)) then
   writeln('Interrupt in use.  Can''t install resident software.')
else
   begin
   TSR_DSEG:= Dseg;           { Save the Data Segment Address for Interrupts }
   TSR_SSEG:= Sseg;           { Save our Stack Segment for Interrupts        }
   GetVector(KBD_INT,Segment,Offset);
   GetVector(TSR_INT,TSR_SEG,TSR_OFF);
   SetVector(TSR_INT,Segment,Offset);
   SetVector(KBD_INT,Cseg,Ofs(ProcessIntr));
   GetVector(TSR_INT,TSR_CS,TSR_IP);
   TSR_SignOn;
   TermStayRes(MemW[Cseg-1:0003]);
   end;
end.