{+-----------------------------------------------------------------------+
 |  Windo 3.0  -  by:  Don Bowen                          Nov. 23, 1986  |
 |                                                                       |
 |       The following procedures OpenWindo & CloseWindo can be used to  |
 |  to manage simple windowing an an MSDOS generic computer.  The two    |
 |  procedures manipulate the video display via system interrupt 10h     |
 |  instead of addressing the hardware directly.  This should make this  |
 |  code portable between machines that are software, but not hardware,  |
 |  compatible.                                                          |
 |       WINDO.INC should be "included" in a main program with the       |
 |  Turbo Pascal include notation.                                       |
 |                                                                       |
 |                         $I DrawBox.inc                                |
 |                         $I IntWindo.inc                               |
 |                                                                       |
 |       The DrawBox procedure MUST come before the windo utilities as   |
 |  OpenWindo uses it to put a boarder around each window.  In addition, |
 |  the global type                                                      |
 |                    Str80 = string[80];                                |
 |                                                                       |
 |  must be declared in the global type declarations.                    |
 |       The procedures can be "called" from a program as follows:       |
 |                                                                       |
 |    1.  OpenWindo( x1, y1, x2, y2, Title, ForColor, BackColor );       |
 |                                                                       |
 |            -  x1,y1 are byte values that denote the window's upper-   |
 |               left corner. x2,y2 describe the lower-right corner      |
 |                                                                       |
 |            -  Title is a string that can contain the window's title.  |
 |               If you specify a title, it will appear, centered, along |
 |               the top of the window.                                  |
 |                                                                       |
 |            -  ForColor & BackColor are byte values that set the       |
 |               window's foreground and background color.               |
 |                                                                       |
 |    2.  CloseWindo;                                                    |
 |                                                                       |
 |            -  The CloseWindo accepts NO parameters as it closes       |
 |               whatever window was most recently opened.               |
 |                                                                       |
 +-----------------------------------------------------------------------+}
const
   MAX_WINDO   = 5;         { max. # of windows that can be opened }
   WINDO_DEPTH : byte = 0;  { count of how many are open           }
type
   ImgType  = array [1..80,1..25] of integer;
   WindoRec = record
                 PreX, PreY,               { cursor position before window }
                 Fg, Bg,                   { forground, background colors  }
                 x1, y1, x2, y2 : byte;    { window dimensions             }
                 Image          : ImgType; { buffer stores covered image   }
              end;
   WindoPtr = ^WindoRec;
var
   WindoList : array [1..MAX_WINDO] of WindoPtr;
   WindoRegs : record             { 8088 CPU registers }
                  case boolean of
                     TRUE  : (AX,BX,CX,DX,BP,SI,DI,DS,ES, FLAGS : integer );
                     FALSE : (AL, AH, BL, BH, CL, CH, DL, DH    : byte    );
               end; { Reg Rec }

procedure GetVidCh( x, y : byte; var ChInt : integer );
   begin
   gotoxy(x,y);
   WindoRegs.AH := 8;         { request read char from screen }
   WindoRegs.BH := 0;         { active display page           }
   Intr($10,WindoRegs);       { cause an interrupt            }
   ChInt   := WindoRegs.AX;   { return character in AL        }
   end;                       {    colors in AH               }

procedure PutVidCh( x, y : byte; ChInt : integer );
   begin
   gotoxy(x,y);
   with WindoRegs do
      begin
      AH := 9;           { request write character/attribute    }
      AL := Lo(ChInt);   { character value to write             }
      BH := 0;           { active display page                  }
      BL := Hi(ChInt);   { byte sensitive char describes colors }
      CX := 1;           { how many characters to write         }
      end;
   Intr($10,WindoRegs);
   end;  { Put Video Ch }

procedure OpenWindo( Wx1, Wy1, Wx2, Wy2 : byte;    { screen dimensions    }
                     Title              : Str80;   { title                }
                     ForColor, BakColor : byte  ); { For/Background Color }
   var
      loopX,
      loopY  : byte;
   begin  { Open window }
   if (WINDO_DEPTH = MAX_WINDO) then
      begin
      writeln('window''s nested too deep !');
      HALT;
      end;
   WINDO_DEPTH := succ(WINDO_DEPTH);
   new(WindoList[WINDO_DEPTH]);
   with WindoList[WINDO_DEPTH]^ do
      begin
      PreX := WhereX;
      PreY := WhereY;
      Fg   := ForColor;
      Bg   := BakColor;
      x1   := Wx1;
      y1   := Wy1;
      x2   := Wx2;
      y2   := Wy2;
      window(1,1,80,25);
      for loopY := y1 to y2 do
         for loopX := x1 to x2 do
            GetVidCh( loopX, loopY, Image[loopX,loopY] );
      TextColor(ForColor);
      TextBackground(BakColor);
      DrawBox(Wx1,Wy1,Wx2,Wy2);
      if (Length(Title) > 0) then
         begin
         if (Length(Title) > x2-x1) then
            Title[0] := chr(pred(x2-x1));
         gotoxy( x1+ succ((x2-x1-Length(Title)) div 2), y1 );
         write(Title);
         end;
      window(succ(Wx1),succ(Wy1),pred(Wx2),pred(Wy2));
      ClrScr;
      end;
   end;   { Open window }

procedure CloseWindo;
   var
      loopX,
      loopY,
      Attr   : byte;
   begin
   if (WINDO_DEPTH > 0) then
      begin
      window(1,1,80,25);
      with WindoList[WINDO_DEPTH]^ do
         begin
         for loopY := y1 to y2 do
            for loopX := x1 to x2 do
               PutVidCh( loopX, loopY, Image[loopX,loopY] );
         if (WINDO_DEPTH > 1) then
            begin
            with WindoList[pred(WINDO_DEPTH)]^ do
               begin
               window(succ(x1),succ(y1),pred(x2),pred(y2));
               TextColor(Fg);
               TextBackGround(Bg);
               end;
            end
         else
            begin
            Attr  := Hi(Image[loopX,loopY]);
            loopX := ( (Attr and 15) + ((Attr shr 7) * Blink) );
            loopY := ( Attr shr 4 and 7 );
            TextColor(loopX);
            TextBackGround(loopY);
            end;
         gotoxy(PreX,PreY);
         end;
      Dispose(WindoList[WINDO_DEPTH]);
      WINDO_DEPTH := pred(WINDO_DEPTH);
      end;
   end;  { Close Windo }